/* $Workfile: OtxRaw.h $   $Revision: 9 $    $Modtime: 12/17/03 5:16p $  */
#ifndef OTX_RAWDATA_H
#define OTX_RAWDATA_H
/*******************************************************************************

  @doc PublicHwApi OtxRawData

  Copyright (C) 1999 - 2003 Odin TeleSystems Inc., All Rights Reserved

  OTX Generic Hardware Driver API

  @module
  OtxRawData | Prototypes for the Logical HDLC Device (Sender/Receiver) Functions

  @contents2
  OtxRawData.h |

  @index func | OtxRawData

  @contents2
  OTX Logical RawData Sender/Receiver Functions - OtxRawDataXXX() | The API functions for the Logical High-level Data Link Controller (HDLC) device

  @head3 OtxRawDataXXX() Function Reference |

  @index func | OtxRawData

*******************************************************************************/

#ifdef __cplusplus
  extern "C" {
#endif

#include "OtxType.h"
#include "OtxDef.h"
#include "OtxDataD.h"

#include "OtxRawD.h"


/*******************************************************************************

  @func
    Initialize the Raw Data Sender.

  @comm
    Start Raw Data Sender operation with by calling the function <f OtxDrvEnable> and stop
    it by calling the function <f OtxDrvDisable>

    @ex Using the Raw Data Sender with a 16 kpbs (two bits) subchannel with 
    bLsbSentFirst==OTX_TRUE: When working with a 16 kbps subchannel the Raw Data Sender will extract
    2 bits from each octet passed to the device and put these bits in 4 consecutive frames. The 2 bits are 
    extracted from the octet according to the figure below: |


     |
     |7 6 5 4 3 2 1 0
     |---+---+---+---+
     |x x|x x|x x|x x|  Octet passed to the Raw Data Sender
     |---------------+
     | ^           ^
     | |           |
     | |           +----- These two bits are sent first
     | |
     | |
     | +----- These two bits are sent last


  @xref
    <f OtxRawDataReceiverInit>
    <f OtxRawDataSendData>
    <e OtxLogicalDeviceTypeE.OTX_LDEVICE_DATA_RAW_SENDER>
    <t OtxDataFormatE>
 
*******************************************************************************/
OTX_RESULT OTX_API OtxRawDataSenderInit(
  IN OTX_HANDLE     hRawDataDevice,   // @parm Handle to logical Raw Data Device
  IN OtxDataFormatE eDataFormat,      // @parm Format of the Raw Data supplied to this device. 
                                      // Use <e OtxDataFormatE.OTX_DATA_FORMAT_UINT8> if no conversion to be applied. 
                                      // Use <e OtxDataFormatE.OTX_DATA_FORMAT_COMPAND8_A_LAW> or <e OtxDataFormatE.OTX_DATA_FORMAT_COMPAND8_U_LAW> 
                                      // to supply 16-bit signed integer values
                                      // in the calls to <f OtxRawDataSendData> and let the device compress these value to 8-bit
                                      // A-law/U-law values before they are placed in the outgoing data stream.

  IN OTX_UINT8      flag,             // @parm Value to send when there is no more Raw Data to send
  IN OTX_BOOL       bLsbSentFirst     // @parm Only applicable for subchannels (pipe capacity of 8, 16, or 32kpbs). Determines in which order bits
                                      // are extracted from the each octet before they are placed into a subchannel pipe. 
                                      // If TRUE the device will send the least significant bits of the octet first, 
                                      // and send the most significant bits last. 
                                      // If FALSE the device will send the most significant bits of the octet first, 
                                      // and send the least significant bits last. 
);


/*******************************************************************************

  @func
    Initializate the HDLC Receiver.

  @comm
    Start HDLC operation with by calling the function <f OtxDrvEnable> and stop
    it by calling the function <f OtxDrvDisable>

    @ex Using the Raw Data Receiver with a 16 kpbs (two bits) subchannel with 
    bLsbReceivedFirst==OTX_TRUE. When working with a 16 kbps subchannel the Raw Data Receiver will extract
    2 bits from 4 consecutive frames and format the 4*2=8 bits into one octet 
    according to the figure below: |
     
     |
     |7 6 5 4 3 2 1 0
     |---+---+---+---+
     |x x|x x|x x|x x|  Octet received by the Raw Data Receiver
     |---------------+
     | ^           ^
     | |           |
     | |           +----- These two bits are the first two bits received 
     | |                  from the subchannel pipe
     | |
     | |
     | +----- These two bits are the last two bits received 
     |        from the subchannel pipe


  @xref
    <f OtxRawDataSenderInit>
    <f OtxRawDataReadData>
    <e OtxLogicalDeviceTypeE.OTX_LDEVICE_DATA_RAW_RECEIVER>
    <t OtxDataFormatE>

*******************************************************************************/
OTX_RESULT OTX_API OtxRawDataReceiverInit(
  IN OTX_HANDLE     hRawDataDevice,   // @parm Handle to logical Raw Data Device
  IN OtxDataFormatE eDataFormat,      // @parm Format of the received Raw Data. 
                                      // Use <e OtxDataFormatE.OTX_DATA_FORMAT_UINT8> if no data conversion to be applied. 
                                      // Use <e OtxDataFormatE.OTX_DATA_FORMAT_COMPAND8_A_LAW> or <e OtxDataFormatE.OTX_DATA_FORMAT_COMPAND8_U_LAW> 
                                      // to expand the 8-bit values from the incoming data stream to 16-bit signed integer values
                                      // retrieved in the <f OtxRawDataReadData> calls.

  IN OTX_UINT8      flag,             // @parm Flag to be extracted if <p bExtractFlag> is TRUE
  IN OTX_BOOL       bExtractFlag,     // @parm if TRUE then received octets that equal the flag value are extracted from the receive data stream
  IN OTX_BOOL       bLsbReceivedFirst // @parm Only applicable for subchannels (pipe capacity of 8, 16, or 32kpbs). If TRUE then the received octets are 
                                      // formatted so that the least significant bits of the octet contains the the first received bits of 
                                      // the subchannel. If FALSE then the received octets are 
                                      // formatted so that the most significant bits of the octet contains the the first received bits of 
                                      // the subchannel;  
);


/*******************************************************************************

  @func
    Sends transparent data.

*******************************************************************************/
OTX_RESULT OTX_API OtxRawDataSendData(
  IN OTX_HANDLE     hRawDataDevice,   // @parm Handle to the Raw Data Sender Device
  IN OTX_UINT8      *sqData,          // @parm Data to be sent
  IN OTX_UINT32     nDataLen,         // @parm Length of the data to be send
  IN OTX_TASK_REF   nTaskRef,         // @parm See <t OTX_TASK_REF>
  IN OTX_TIME       nWaitMaxMs        // @parm See <t OTX_TIME>
);


/*******************************************************************************

  @func
    Flushes (empties) the queue of raw data octets waiting to be transmitted (if any).
  @xref
    <f OtxRawDataSendData>

*******************************************************************************/
OTX_RESULT OTX_API OtxRawDataSenderFlush(
  IN OTX_HANDLE     hRawDataDevice    // @parm Handle to the Raw Data Sender Device
);


/*******************************************************************************

  @func
    Read data received by the Raw Data receiver

  @rdesc Returns one of the following values:

    @flag OTX_S_OK | Raw Data Data was fetched
    @flag OTX_S_FALSE | No Raw Data Data available at this moment
    @flag OTX_E_BUFFER_TO_SMALL | The supplied user data buffer is too small to fit the received data

  @xref
    <f OtxRawDataReceiverInit>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_DATA_RAW_RECEIVER>
    <nl><f OtxRawDataSendData>

*******************************************************************************/
OTX_RESULT OTX_API OtxRawDataReadData(
  IN  OTX_HANDLE    hRawDataDevice,   // @parm Handle to the Raw Data Receiver Device
  OUT OTX_UINT8     *pDataBuf,        // @parm Data buffer for the received data
  IN  OTX_UINT32    nBufLen,          // @parm Length of the user data buffer
  OUT OTX_UINT32    *pnFrameLen,      // @parm Length of the received raw data buffer (number of octets)
  OUT OTX_BOOL      *pbFrameEnd       // @parm Set to OTX_TRUE if a flag (as specified in <f OtxRawDataReceiverInit>) followed the received data frame (valid if OTX_TRUE if <f OtxRawDataReceiverInit> was initialized with bExtractFlag=OTX_TRUE)
);


/*******************************************************************************

  @func
    Load an idle pattern for the Raw Data Sender devicie and begin transmission. The Idle Pattern
    is a sequence of data which will be send continuosly always when there is no
    actual user data to be send. User data can be send with <f OtxRawDataSendData> function
    on which case the user data frame in interleaved between two consequtive idle
    patterns. After the sending of user data has been completed,
    the Raw Data device will resume the sending of the idle pattern.
  @xref
   <f OtxRawDataStopIdlePattern>

*******************************************************************************/
OTX_RESULT OTX_API OtxRawDataStartIdlePattern(
  IN OTX_HANDLE     hRawDataDevice,   // @parm Handle to the HDLC Device
  IN OTX_UINT8      *sqData,          // @parm Data to be sent
  IN OTX_UINT32     nDataLen          // @parm Length of the data to be send
);


/*******************************************************************************

  @func
    Stop sending of idle pattern on the specified Raw Data Sender device.
  @xref
   <f OtxRawDataStartIdlePattern>

*******************************************************************************/
OTX_RESULT OTX_API OtxRawDataStopIdlePattern(
  IN OTX_HANDLE     hRawDataDevice    // @parm Handle to the HDLC Device
);



#ifdef __cplusplus
  } // extern "C"
#endif


/*******************************************************************************

  Copyright (C) Odin TeleSystems Inc., 1999 - 2003

  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

*******************************************************************************/
#endif // OTX_RAWDATA_H
